using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;
using BibliotecaComum;

namespace AmbienteDeDesenvolvimento
{
    public partial class PrincForm : Form
    {
        Projeto _projeto = null;
        string _titulo = null;

        public PrincForm(string[] args)
        {
            if (args.Length > 0)
            {
                string end = args[0];
                if (File.Exists(end)) end = Path.GetDirectoryName(end);
                _projeto = new Projeto(end);
            }
            Config.Iniciar();
            InitializeComponent();
        }

        private void criarAba(Control conteudo, string titulo)
        {
            TabPage aba = new TabPage(titulo);
            aba.Controls.Add(conteudo);
            conteudo.Dock = DockStyle.Fill;
            editorAbas.TabPages.Add(aba);
            editorAbas.SelectedTab = aba;
        }

        private void fecharAbaAtual()
        {
            if (editorAbas.SelectedTab != null)
            {
                if (editorAbas.SelectedTab.Text.Contains("*"))
                {
                    switch (MessageBox.Show("Deseja salvar alteraes no projeto antes de fechar este arquivo?", "HUSIS", MessageBoxButtons.YesNoCancel))
                    {
                        case DialogResult.Cancel:
                            return;
                        case DialogResult.No:
                            break;
                        case DialogResult.Yes:
                            salvar();
                            break;
                        default:
                            break;
                    }
                }
                editorAbas.TabPages.Remove(editorAbas.SelectedTab);
            }
        }

        private TabPage buscar(string titulo)
        {
            foreach (TabPage aba in editorAbas.TabPages)
            {
                if (aba.Text.Replace(" *", "").ToLower() == titulo.ToLower())
                {
                    return aba;
                }
            }
            return null;
        }

        private bool buscarESelecionar(string titulo)
        {
            TabPage aba = buscar(titulo);
            if (aba != null)
            {
                editorAbas.SelectedTab = aba;
                return true;
            }
            return false;
        }

        private void editar(string arquivo)
        {
            if (buscarESelecionar(arquivo)) return;
            EditorControle editor = new EditorControle(_projeto, arquivo);
            criarAba(editor, arquivo);
        }

        private void editarProjeto()
        {
            if (buscarESelecionar("Conf. do Projeto")) return;
            ProjetoControle pc = new ProjetoControle(_projeto);
            criarAba(pc, "Conf. do Projeto");
        }

        private void carregaProjeto(string diretorio)
        {
            _projeto = new Projeto(diretorio);
            carregaProjeto();
        }

        private void carregaProjeto()
        {
            Config.UltimoProjeto = _projeto.EnderecoDiretorio;
            editorAbas.TabPages.Clear();
            recarregarProjeto();
            Text = _titulo + " - " + _projeto.EnderecoDiretorio;
        }

        private void recarregarProjeto()
        {
            itensLista.Items.Clear();
            itensLista.Items.Add(_projeto.ArquivoConfiguracao, 3);
            itensLista.Items[0].Selected = true;
            menus(true);
            foreach (string arquivo in _projeto.Arquivos)
            {
                int tipo = 0;
                switch (_projeto.Extensao(arquivo))
                {
                    case "s":
                    case "asm":
                        tipo = 0;
                        break;
                    case "tra":
                        tipo = 1;
                        break;
                    default:
                        tipo = 4;
                        break;
                }
                itensLista.Items.Add(arquivo, tipo);
            }
        }

        private void abrirProjeto()
        {
            if (Directory.Exists(Config.UltimoProjeto))
            {
                abrirProjetoJanela.SelectedPath = Config.UltimoProjeto;
            }
            if (abrirProjetoJanela.ShowDialog() == DialogResult.OK)
            {
                if (_projeto != null)
                {
                    switch (MessageBox.Show("Deseja salvar alteraes antes de fechar?", "HUSIS", MessageBoxButtons.YesNoCancel))
                    {
                        case DialogResult.Cancel:
                            return;
                        case DialogResult.No:
                            break;
                        case DialogResult.Yes:
                            salvar();
                            break;
                        default:
                            break;
                    }
                }
                carregaProjeto(abrirProjetoJanela.SelectedPath);
            }
        }

        private void menus(bool exibir)
        {
            novoArquivoAssemblyMenu.Visible = exibir;
            novaTraduoMenu.Visible = exibir;
            salvarBotao.Visible = exibir;
            compilarBotao.Visible = exibir;
            distribuirBotao.Visible = exibir;
            publicarBotao.Visible = exibir;
            importarMenu.Visible = exibir;
            excluirMenu.Visible = exibir;
        }

        private void salvar()
        {
            if (_projeto == null) return;
            statusRotulo.Text = "Salvando . . .";
            _projeto.Salvar();
            foreach (TabPage aba in editorAbas.TabPages)
            {
                foreach (Control controle in aba.Controls)
                {
                    if (controle is EditorControle)
                    {
                        EditorControle ec = (EditorControle)controle;
                        ec.Salvar();
                    }
                    else if (controle is ProjetoControle)
                    {
                        ProjetoControle pc = (ProjetoControle)controle;
                        pc.Salvar();
                    }
                }
            }
            statusRotulo.Text = "";
        }

        private void PrincForm_Load(object sender, EventArgs e)
        {
            _titulo = Text;
            WindowState = FormWindowState.Maximized;
            menus(false);
            carregarTraducao();
            if (_projeto == null)
                novoProjetoMenu.PerformClick();
            else
                carregaProjeto();
        }

        private void trad(ToolStripItem obj, string txt)
        {
            obj.Text = txt;
        }

        private void trad(TabPage obj, string txt)
        {
            obj.Text = txt;
        }

        private void trad(Control obj, string txt)
        {
            obj.Text = txt;
        }

        private void carregarTraducao()
        {
            Traducao.Iniciar(Config.Traducao);
            trad(compilarBotao, Traducao.Compilar);
            trad(publicarBotao, Traducao.Publicar);
            trad(distribuirBotao, Traducao.Distribuir);
            trad(salvarBotao, Traducao.Salvar);
            trad(importarMenu, Traducao.Importar);
            trad(excluirMenu, Traducao.Excluir);
            trad(abrirProjetoMenu, Traducao.AbrirProjeto);
            trad(novoMenu, Traducao.Novo);
            trad(novoProjetoMenu, Traducao.NovoProjeto);
            trad(novoArquivoAssemblyMenu, Traducao.NovoArquivoAssembly);
            trad(novaTraduoMenu, Traducao.NovaTraducao);
            trad(fecharAbaBotao, Traducao.FecharAba);
            trad(errosAba, Traducao.Erros);
            trad(ultimaCompilacaoAba, Traducao.UltimaCompilacao);
            trad(projetoGrupo, Traducao.Projeto);
            trad(configuracoesBotao, Traducao.Configuracoes);
        }

        private void itensLista_ItemActivate(object sender, EventArgs e)
        {
            string arquivo = itensLista.SelectedItems[0].Text;
            if (arquivo == _projeto.ArquivoConfiguracao)
            {
                editarProjeto();
            }
            else
            {
                editar(arquivo);
            }
        }

        private void salvarBotao_Click(object sender, EventArgs e)
        {
            salvar();
        }

        private void novoArquivoAssemblyMenu_Click(object sender, EventArgs e)
        {
            PerguntaForm pf = new PerguntaForm(Traducao.NovoArquivoAssemblyTexto);
            if (pf.ShowDialog() == DialogResult.OK)
            {
                string arquivo = Path.GetFileNameWithoutExtension(pf.Resposta) + "." + _projeto.ExtensaoAssembly;
                foreach (string existente in _projeto.Arquivos)
                {
                    if (existente.ToLower() == arquivo.ToLower())
                    {
                        MessageBox.Show("Arquivo j existe");
                        return;
                    }
                }
                _projeto.Adicionar(arquivo, AcaoArquivo.CompilarAssembly);
                salvar();
                recarregarProjeto();
            }
        }

        private void PrincForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_projeto == null) return;
            bool perguntar = false;
            foreach (TabPage aba in editorAbas.TabPages)
            {
                if (aba.Text.EndsWith("*"))
                {
                    perguntar = true;
                    break;
                }
            }
            if (perguntar)
            {
                switch (MessageBox.Show("Deseja salvar alteraes antes de fechar?", "HUSIS", MessageBoxButtons.YesNoCancel))
                {
                    case DialogResult.Cancel:
                        e.Cancel = true;
                        break;
                    case DialogResult.No:
                        break;
                    case DialogResult.Yes:
                        salvar();
                        break;
                    default:
                        break;
                }
            }
        }

        private void fecharAbaBotao_Click(object sender, EventArgs e)
        {
            fecharAbaAtual();
        }

        private void excluirMenu_Click(object sender, EventArgs e)
        {
            string arquivo = itensLista.SelectedItems[0].Text;
            if (arquivo == _projeto.ArquivoConfiguracao)
            {
                MessageBox.Show(Traducao.NaoFoiPossivelRemover);
                return;
            }
            if (MessageBox.Show(string.Format(Traducao.ConfirmaExclusao, arquivo), "", MessageBoxButtons.YesNoCancel) == DialogResult.Yes)
            {
                if (buscarESelecionar(arquivo))
                {
                    fecharAbaAtual();
                }
                _projeto.Remover(arquivo);
                recarregarProjeto();
            }
        }

        private void novoProjetoMenu_Click(object sender, EventArgs e)
        {
            NovoProjetoForm npf = new NovoProjetoForm();
            if (npf.ShowDialog() == DialogResult.OK)
            {
                if (_projeto != null)
                {
                    switch (MessageBox.Show("Deseja salvar alteraes antes de fechar?", "HUSIS", MessageBoxButtons.YesNoCancel))
                    {
                        case DialogResult.Cancel:
                            return;
                        case DialogResult.No:
                            break;
                        case DialogResult.Yes:
                            salvar();
                            break;
                        default:
                            break;
                    }
                }
                _projeto = npf.Projeto;
                carregaProjeto();
            }
        }

        private void abrirProjetoMenu_Click(object sender, EventArgs e)
        {
            abrirProjeto();
        }

        private void importarMenu_Click(object sender, EventArgs e)
        {
            if (importarJanela.ShowDialog() == DialogResult.OK)
            {
                string arquivo = Path.GetFileName(importarJanela.FileName);
                PerguntaForm pf = new PerguntaForm("Informe o nome do arquivo dentro do projeto: (O projeto ser salvo para poder adicionar este arquivo)", arquivo);
                if (pf.ShowDialog() == DialogResult.OK)
                {
                    arquivo = pf.Resposta;
                    AcaoArquivo acao = AcaoArquivo.Nenhuma;
                    switch (_projeto.Extensao(pf.Resposta))
                    {
                        case "s":
                        case "asm":
                        case "inc":
                            acao = AcaoArquivo.CompilarAssembly;
                            break;
                        case "tra":
                            acao = AcaoArquivo.CompilarTraducao;
                            break;
                    }
                    _projeto.Adicionar(arquivo, acao);
                    byte[] binario = File.ReadAllBytes(importarJanela.FileName);
                    File.WriteAllBytes(_projeto.Caminho(arquivo), binario);
                    salvar();
                    recarregarProjeto();
                }
                else
                {
                    MessageBox.Show("Importao cancelada.");
                }
            }
        }

        private void marcarErros(string tipo, Color fundo, Color texto)
        {
            int inicio = 0;
            while (inicio >= 0)
            {
                inicio = errosCampo.Find(tipo, inicio, RichTextBoxFinds.WholeWord);
                if (inicio >= 0)
                {
                    inicio = errosCampo.SelectionLength + inicio;
                    errosCampo.SelectionColor = texto;
                    errosCampo.SelectionBackColor = fundo;
                }
            } 
        }

        private void compilarMenu_Click(object sender, EventArgs e)
        {
            salvar();
            statusRotulo.Text = "Compilando . . .";
            CompilarForm cf = new CompilarForm(_projeto);
            if (cf.ShowDialog() != DialogResult.OK)
            {
                MessageBox.Show("Foram encontrados erros durante a compilao.");
                mensagensAbas.SelectedTab = errosAba;
                statusRotulo.Text = "Compilao encontrou erros.";
            }
            else
            {
                statusRotulo.Text = "Compilado com sucesso.";
            }
            errosCampo.SuspendLayout();
            errosCampo.Text = cf.Retorno;
            errosCampo.SelectAll();
            errosCampo.SelectionColor = SystemColors.ControlText;
            errosCampo.SelectionBackColor = SystemColors.Control;
            ultimaCompilacaoCampo.Text = cf.Compilacao;
            marcarErros("Error:", Color.Red, Color.White);
            marcarErros("Erro:", Color.Red, Color.White);
            marcarErros("Warning:", Color.OrangeRed, Color.White);
            marcarErros("Alerta:", Color.OrangeRed, Color.White);
            marcarErros("Abort:", Color.Red, Color.White);
            marcarErros("Cancelado:", Color.Red, Color.White);
            errosCampo.Select(0, 0);
            errosCampo.ScrollToCaret();
            errosCampo.ResumeLayout();
            this.Focus();
        }

        private void configuracoesBotao_Click(object sender, EventArgs e)
        {
            ConfigForm cf = new ConfigForm();
            cf.ShowDialog();
        }
    }
}