using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace BibliotecaComum
{
    public class ArquivoIni
    {
        string _endereco;

        string[] _bruto;

        Dictionary<string, Dictionary<string, string>> _dados = new Dictionary<string, Dictionary<string, string>>();

        public ArquivoIni(string endereco)
        {
            if (File.Exists(endereco))
            {
                _bruto = File.ReadAllLines(endereco);
            }
            else
            {
                _bruto = new string[] { };
            }
            _endereco = endereco;
            processarBruto();
        }

        private void processarBruto()
        {
            _dados.Clear();
            string segmentoAtual = "";
            int nro = 0;
            foreach (string linha in _bruto)
            {
                nro++;
                if (linha.Trim().StartsWith("["))
                {
                    segmentoAtual = linha.Trim().TrimStart('[').TrimEnd(']');
                    if (!_dados.ContainsKey(segmentoAtual))
                    {
                        _dados.Add(segmentoAtual, new Dictionary<string, string>());
                    }
                }
                else
                {
                    string[] trechos = linha.Split('=');
                    if (trechos.Length != 2)
                    {
                        throw new FormatException("Formato invlido na linha " + nro.ToString().Trim());
                    }
                    _dados[segmentoAtual].Add(trechos[0], trechos[1]);
                }
            }
        }

        private void gerarBruto()
        {
            List<string> novo = new List<string>();
            foreach (KeyValuePair<string, Dictionary<string, string>> segmento in _dados)
            {
                novo.Add("[" + segmento.Key + "]");
                foreach (KeyValuePair<string, string> item in segmento.Value)
                {
                    if (item.Key != null & item.Value != null)
                    {
                        novo.Add(item.Key + "=" + item.Value);
                    }
                }
            }
            _bruto = novo.ToArray();
        }

        public void ImportarBruto(string[] bruto)
        {
            _bruto = bruto;
            processarBruto();
        }

        public string[] ExportarBruto()
        {
            string[] ret = new string[_bruto.Length];
            Array.Copy(_bruto, ret, _bruto.Length);
            return ret;
        }

        public ArquivoIni Duplicar(string novoEndereco)
        {
            ArquivoIni novo = new ArquivoIni(novoEndereco);
            novo.ImportarBruto(_bruto);
            return novo;
        }

        public string[] Listar(string segmento)
        {
            if (_dados.ContainsKey(segmento))
            {
                List<string> ret = new List<string>();
                ret.AddRange(_dados[segmento].Keys);
                return ret.ToArray();
            }
            return new string[] { };
        }

        public string Ler(string segmento, string chave, string definirValorSeNaoExiste)
        {
            if (_dados.ContainsKey(segmento))
            {
                if (_dados[segmento].ContainsKey(chave))
                {
                    return _dados[segmento][chave];
                }
            }
            Gravar(segmento, chave, definirValorSeNaoExiste);
            return definirValorSeNaoExiste;
        }

        public void Gravar(string segmento, string chave, string valor)
        {
            if (!_dados.ContainsKey(segmento))
            {
                _dados.Add(segmento, new Dictionary<string, string>());
            }
            if (!_dados[segmento].ContainsKey(chave))
            {
                _dados[segmento].Add(chave, valor);
            }
            else
            {
                _dados[segmento][chave] = valor;
            }
        }

        public void Remover(string segmento, string chave)
        {
            if (!_dados.ContainsKey(segmento))
            {
                _dados.Add(segmento, new Dictionary<string, string>());
            }
            if (_dados[segmento].ContainsKey(chave))
            {
                _dados[segmento].Remove(chave);
            }
        }

        public void SalvarArquivo()
        {
            gerarBruto();
            File.WriteAllLines(_endereco, _bruto);
        }
    }

}
