using System;
using System.Collections.Generic;
using System.Text;

namespace Roteador
{
    class PacoteHCCom : PacoteBase
    {
        public PacoteHCCom()
        {
        }

        public PacoteHCCom(EnderecoHCCom origem, EnderecoHCCom destino, ushort porta, ushort conexao, ushort posicao, ushort comando, byte[] dados)
        {
            Origem = origem;
            Destino = destino;
            Porta = porta;
            Conexao = conexao;
            Posicao = posicao;
            Comando = comando;
            Dados = dados;
        }

        public PacoteHCCom(string origem, string destino, ushort porta, ushort conexao, ushort posicao, ushort comando, byte[] dados)
        {
            Origem = new EnderecoHCCom(origem);
            Destino = new EnderecoHCCom(destino);
            Porta = porta;
            Conexao = conexao;
            Posicao = posicao;
            Comando = comando;
            Dados = dados;
        }

        public PacoteHCCom(string origem, string destino, PortasHCCom porta, ushort conexao, ushort posicao, ushort comando, byte[] dados)
        {
            Origem = new EnderecoHCCom(origem);
            Destino = new EnderecoHCCom(destino);
            Porta = (ushort)porta;
            Conexao = conexao;
            Posicao = posicao;
            Comando = comando;
            Dados = dados;
        }

        public PacoteHCCom(PacoteSimples pacote)
        {
            ImportarPacoteSimples(pacote);
        }

        private EnderecoHCCom _origem;

        public EnderecoHCCom Origem
        {
            get { return _origem; }
            set { _origem = value; }
        }

        private EnderecoHCCom _destino;

        public EnderecoHCCom Destino
        {
            get { return _destino; }
            set { _destino = value; }
        }

        private ushort _porta;

        public ushort Porta
        {
            get { return _porta; }
            set { _porta = value; }
        }

        private ushort _conexao;

        public ushort Conexao
        {
            get { return _conexao; }
            set { _conexao = value; }
        }


        private ushort _posicao;

        public ushort Posicao
        {
            get { return _posicao; }
            set { _posicao = value; }
        }

        private ushort _comando;

        public ushort Comando
        {
            get { return _comando; }
            set { _comando = value; }
        }


        public override PacoteSimples ExportarPacoteSimples()
        {
            PacoteSimples ps = new PacoteSimples();
            Fluxo fluxo = new Fluxo(ps);
            ps.Dados = new byte[18 + Tamanho];
            fluxo.Byte((byte)'H');
            fluxo.Byte((byte)'C');
            Origem.GravarEm(fluxo);
            Destino.GravarEm(fluxo);
            fluxo.UShort(Conexao);
            fluxo.UShort(Porta);
            fluxo.UShort(Comando);
            fluxo.UShort((ushort)Tamanho);
            if (Tamanho > 1024) throw new OverflowException("Pacote maior que 1024 bytes de contedo");
            fluxo.Bytes(Dados);
            fluxo.Posicao = 0;
            ushort checagem = 0;
            while (fluxo.Falta > 1)
            {
                ushort numero = fluxo.UShort();
                checagem = (ushort)((checagem + numero) & 0xffff 
                    + (((checagem + numero) & 0x10000) > 0 ? 1 : 0));
            }
            if (fluxo.Falta == 1)
            {
                ushort numero = (ushort)fluxo.Byte();
                checagem = (ushort)((checagem + numero) & 0xffff
                    + (((checagem + numero) & 0x10000) > 0 ? 1 : 0));
            }
            fluxo.UShort(checagem);
            return ps;
        }

        public override void ImportarPacoteSimples(PacoteSimples pacote)
        {
            Fluxo fluxo = new Fluxo(pacote);
            ushort checagem = 0;
            while ((fluxo.Falta - 2) > 1)
            {
                ushort numero = fluxo.UShort();
                checagem = (ushort)((checagem + numero) & 0xffff
                    + (((checagem + numero) & 0x10000) > 0 ? 1 : 0));
            }
            if (fluxo.Falta == 3)
            {
                ushort numero = (ushort)fluxo.Byte();
                checagem = (ushort)((checagem + numero) & 0xffff
                    + (((checagem + numero) & 0x10000) > 0 ? 1 : 0));
            }
            ushort checagemPacote = fluxo.UShort();
            if (checagem != checagemPacote)
            {
                throw new InvalidCastException("Pacote corrompido (Chegagem)");
            }
            fluxo.Posicao = 0;
            if (fluxo.Byte() != (byte)'H' |  fluxo.Byte() != (byte)'C')
            {
                throw new InvalidCastException("Pacote corrompido (Identificador)");
            }
            Origem = new EnderecoHCCom();
            Origem.LerDe(fluxo);
            Destino = new EnderecoHCCom();
            Destino.LerDe(fluxo);
            Conexao = fluxo.UShort();
            Porta = fluxo.UShort();
            Comando = fluxo.UShort();
            int tam = fluxo.UShort();
            Dados = fluxo.Bytes(tam);
        }
    }
}
